diff -burN kernel-source-2.4.21/usr/src/linux-2.4.21-138/drivers/audit/control.c linux-2.4.21-138/drivers/audit/control.c
--- kernel-source-2.4.21/usr/src/linux-2.4.21-138/drivers/audit/control.c	Thu Oct 30 17:37:35 2003
+++ linux-2.4.21-138/drivers/audit/control.c	Tue Jan 27 15:28:42 2004
@@ -63,7 +63,7 @@
 int				audit_all_processes = 0;
 unsigned int			audit_max_messages = 1024;
 int				audit_allow_suspend = 1;
-int				audit_paranoia = 1;
+int				audit_paranoia = 0;
 
 static int	__audit_attach(struct task_struct *, int, struct aud_process *);
 static void	audit_attach_all(void);
@@ -199,9 +199,9 @@
 	if (!capable(CAP_SYS_ADMIN))
 		return -EPERM;
 
-	down_write(&audit_lock);
 	switch (cmd) {
 	case AUIOCIAMAUDITD:
+		down_write(&audit_lock);
 		error = audit_msg_enable();
 		if (error >= 0) {
 			printk(KERN_NOTICE
@@ -214,41 +214,64 @@
 		}
 		if (audit_all_processes)
 			audit_attach_all();
+		up_write(&audit_lock);
 		break;
 	case AUIOCATTACH:
+		down_write(&audit_lock);
 		/* Attach process. If we're rhe audit daemon,
 		 * suspend auditing for us. */
 		error = audit_attach(ctx->reader);
+		up_write(&audit_lock);
 		break;
 	case AUIOCDETACH:
+		down_write(&audit_lock);
 		error = audit_detach();
+		up_write(&audit_lock);
 		break;
 	case AUIOCSUSPEND:
+		down_write(&audit_lock);
 		error = audit_suspend();
+		up_write(&audit_lock);
 		break;
 	case AUIOCRESUME:
+		down_write(&audit_lock);
 		error = audit_resume();
+		up_write(&audit_lock);
 		break;
 	case AUIOCCLRPOLICY:
+		down_write(&audit_lock);
 		error = audit_policy_clear();
+		up_write(&audit_lock);
 		break;
 	case AUIOCCLRFILTER:
+		down_write(&audit_lock);
 		error = audit_filter_clear();
+		up_write(&audit_lock);
 		break;
 	case AUIOCSETFILTER:
+		down_write(&audit_lock);
 		error = audit_filter_add((void *) arg);
+		up_write(&audit_lock);
 		break;
 	case AUIOCSETPOLICY:
+		down_write(&audit_lock);
 		error = audit_policy_set((void *) arg);
+		up_write(&audit_lock);
 		break;
 	case AUIOCSETAUDITID:
+		down_write(&audit_lock);
 		error = audit_setauditid();
+		up_write(&audit_lock);
 		break;
 	case AUIOCLOGIN:
+		down_read(&audit_lock);
 		error = audit_login((void *) arg);
+		up_read(&audit_lock);
 		break;
 	case AUIOCUSERMESSAGE:
+		down_read(&audit_lock);
 		error = audit_user_message((void *) arg);
+		up_read(&audit_lock);
 		break;
 
 	default:
@@ -256,7 +279,6 @@
 		break;
 	}
 
-	up_write(&audit_lock);
 	DPRINTF("done, result=%d\n", error);
 	return (error);
 }
@@ -495,7 +517,7 @@
 {
 	struct aud_syscall_data	*sc;
 	struct aud_process	*pinfo;
-	int			error;
+	int			error = -EL3RST; /* initialized below */
 
 	/* Oops, process not attached? */
 	if ((pinfo = current->audit) == NULL) {
@@ -577,19 +599,29 @@
 		 *		assembling the audit message, the process image
 		 *		is gone.
 		 * unlink	Resolve pathnames before file is gone
+		 * close        Resolve pathname for fd before it is closed
 		 * rename	First pathname needs to be resolved before
 		 *		the call; afterwards it's gone already.
 		 * chroot	Pathname must be interpreted relative to
 		 *		original fs->root.
+		 * sendfile     changes the *offset argument
 		 */
 		switch (sc->major) {
+			void *p;
 		case __NR_execve:
 		case __NR_unlink:
 		case __NR_chroot:
+		case __NR_close:
 			error = audit_copy_arguments(sc);
 			break;
 		case __NR_rename:
-			error = PTR_ERR(audit_get_argument(sc, 0));
+			p = audit_get_argument(sc, 0);
+			if (IS_ERR(p)) error = PTR_ERR(p);
+			break;
+		case __NR_sendfile:
+			p = audit_get_argument(sc, 2);
+			if (IS_ERR(p)) error = PTR_ERR(p);
+			break;
 		}
 	}
 
